function [ds ext] = eqns_hangerControl_022213(t,s)

global  G1 G2 G3 md mh mb g Kp Kd Ki rd rh rb rhm Rd Ckt1 Ckt2 bt bf k_kmmt b_kmmt k_kmm b_kmm I1 I2 I3 Input_Ang_acc IAV1 IAV2 omega t_acc t_vel

qkmm=s(1);
qslop=s(2);
q1=s(3);
q2=s(4);
q3=s(5);
q4=s(6);
q5=s(7);
q6=s(8);
p_theta1=s(9);
p_theta2=s(10);
p_theta3=s(11);
p_x1=s(12);
p_y1=s(13);
p_x2=s(14);
p_y2=s(15);
p_x3=s(16);
p_y3=s(17);
theta1=s(18);
theta2=s(19);
theta3=s(20);
in_angle=s(21);
integral_theta3 = s(22);

%% Sinusoidal input
tstart = 10.2;
tend = tstart + 3.79*(2*3.14/omega); % run for 3 and 3/4 periods
tstart_ramp = 1.5 + tend;
if t <= tstart
    Input_Ang_Vel = 0;
elseif t > tstart  && t < tend
    Input_Ang_Vel = IAV1*sin(omega*(t-tstart));
elseif t>=tend && t < tstart_ramp
    Input_Ang_Vel = 0;
end
%% Ramp input
tstop_ramp = tstart_ramp + t_acc;
if t <= tstart_ramp && t>tend
    Input_Ang_Vel = 0;
elseif t > tstart_ramp && t <= tstop_ramp
    Input_Ang_Vel = Input_Ang_acc*(t-tstart_ramp);
elseif t > tstop_ramp 
    Input_Ang_Vel = 0;
end
% %% Step input
% tstart = 0.2;
% tend = tstart + t_vel;
% if t <= tstart
%     Input_Ang_Vel = 0;
% elseif t > tstart  && t<tend
%     Input_Ang_Vel = IAV2;
% elseif t>=tend
%     Input_Ang_Vel = 0;
% end
%% State equations for main system
% Define velocities to be used in state equations
x2_dot = p_x2/mh; y2_dot = p_y2/mh;
x1_dot = p_x1/md; y1_dot = p_y1/md;
x3_dot = p_x3/mb; y3_dot = p_y3/mb;
theta1_dot = p_theta1/I1; theta2_dot = p_theta2/I2; theta12_dot = theta1_dot - theta2_dot; theta3_dot = p_theta3/I3; theta23_dot = theta2_dot - theta3_dot;
% Create control signal
tau_m = -(-Kp*theta3 - Kd*theta3_dot - Ki*integral_theta3);
if tau_m>5.2*g/100
    tau_m = 5.2*g/100;
end  
if tau_m<-5.2*g/100
    tau_m = -5.2*g/100;
end 
% Displacement state equations
qkmm_dot = Input_Ang_Vel - theta1_dot;
qslop_dot = theta12_dot;
q1_dot = -rd*sin(theta1)*theta1_dot - x1_dot;
q2_dot = rd*cos(theta1)*theta1_dot - y1_dot;
q3_dot = -rh*sin(theta2)*theta2_dot - Rd*sin(theta1)*theta1_dot - x2_dot;
q4_dot = rh*cos(theta2)*theta2_dot + Rd*cos(theta1)*theta1_dot - y2_dot;
q5_dot = -rb*sin(theta3)*theta3_dot - rhm*sin(theta2)*theta2_dot - Rd*sin(theta1)*theta1_dot - x3_dot;
q6_dot = rb*cos(theta3)*theta3_dot + rhm*cos(theta2)*theta2_dot + Rd*cos(theta1)*theta1_dot - y3_dot;
% Constraint forces
F1 = (k_kmm*G1)*q1 + b_kmm*q1_dot;    %Fx,1
F2 = k_kmm*q2 + b_kmm*q2_dot;         %Fy,1
F3 = (k_kmm*G2)*q3 + b_kmm*q3_dot;    %Fx,2
F4 = k_kmm*q4 + b_kmm*q4_dot;         %Fy,2
F5 = (k_kmm*G3)*q5 + b_kmm*q5_dot;    %Fx,3
F6 = k_kmm*q6 + b_kmm*q6_dot;         %Fy,3
tau_slop = ((Ckt1*qslop^3 + Ckt2*qslop) + bt*qslop_dot);
tau_kmm = k_kmmt*qkmm + b_kmmt*qkmm_dot;
% Momenta state equations
p_x3_dot = mb*g + F5;
p_y3_dot = F6;
p_x2_dot = mh*g + F3;
p_y2_dot = F4;
p_x1_dot = md*g + F1;
p_y1_dot = F2;
p_theta1_dot =  tau_kmm - tau_slop + rd*sin(theta1)*F1 - rd*cos(theta1)*F2 + Rd*sin(theta1)*F3 - Rd*cos(theta1)*F4 + Rd*sin(theta1)*F5 - Rd*cos(theta1)*F6;
p_theta2_dot = tau_slop + (tau_m - bf*theta23_dot) + rh*sin(theta2)*F3 - rh*cos(theta2)*F4 + rhm*sin(theta2)*F5 - rhm*cos(theta2)*F6;
p_theta3_dot = -(tau_m - bf*theta23_dot) + rb*sin(theta3)*F5 - rb*cos(theta3)*F6;
%% Extra ouputs
ext(1) = tau_m;                                                         %N-m, Control torque
ext(2) = Input_Ang_Vel; 
ext(3) = F1; 
ext(4) = F2; 
ext(5) = F3; 
ext(6) = F4; 
ext(7) = F5; 
ext(8) = F6; 
ext(9) = rd*cos(theta1);   %v_yd
ext(10) = -rd*sin(theta1); %v_xd
ext(11) = Rd*cos(theta1) + rh*cos(theta2);  %v_yh
ext(12) = -Rd*sin(theta1) - rh*sin(theta2); %v_xh 
ext(13) = Rd*cos(theta1) + rhm*cos(theta2) + rb*cos(theta3);   %v_yb
ext(14) = -Rd*sin(theta1) - rhm*sin(theta2) - rb*sin(theta3);  %v_xb
ext(15) = in_angle;
ext(16) = tau_slop;

%% Euler integration
%snew = [delta_alpha+delta_alpha_dot*dt, delta_z+delta_z_dot*dt, delta_y+delta_y_dot*dt, p_z2+p_z2_dot*dt, p_y2+p_y2_dot*dt, p_z1+p_z1_dot*dt,  p_y1+p_y1_dot*dt, p_alpha+p_alpha_dot*dt, p_beta+p_beta_dot*dt, alpha+alpha_dot*dt, beta+beta_dot*dt];

%% Derivatives
ds = [qkmm_dot; qslop_dot; q1_dot; q2_dot; q3_dot; q4_dot; q5_dot; q6_dot; p_theta1_dot; p_theta2_dot; p_theta3_dot; p_x1_dot; p_y1_dot; p_x2_dot; p_y2_dot; p_x3_dot; p_y3_dot; theta1_dot; theta2_dot; theta3_dot; Input_Ang_Vel; theta3];
% qkmm=s(1);
% qslop=s(2);
% q1=s(3);
% q2=s(4);
% q3=s(5);
% q4=s(6);
% q5=s(7);
% q6=s(8);
% p_theta1=s(9);
% p_theta2=s(10);
% p_theta3=s(11);
% p_x1=s(12);
% p_y1=s(13);
% p_x2=s(14);
% p_y2=s(15);
% p_x3=s(16);
% p_y3=s(17);
% theta1=s(18);
% theta2=s(19);
% theta3=s(20);
% in_angle=s(21);
