%function [base_angle] = master_hC_022213(fin,tspan)

clear all 
close all
clc

tic

global  G1 G2 G3 md mh mb g Kp Kd Ki rd rh rb rhm Rd Ckt1 Ckt2 bt bf k_kmmt b_kmmt k_kmm b_kmm I1 I2 I3 Input_Ang_acc IAV1 IAV2 omega t_acc t_vel

%% Define constants
% Systems parameters
    % Angular velocity input params
    t_acc = .4;
    t_vel = .8;
    Input_Ang_acc = 2*(45*3.14159/180)/(t_acc^2); %rad/s/s
    IAV1 = 235*3.14159/180;
    IAV2 = 45*3.14159/180/t_vel;
    omega = 14;
% System masses, inertias and lengths
    md = 3320/2/1000; % kg, mass of the upper vehicle (battery and all flight equip such as rotors, etc)
    mh = md*.25; %kg, mass of the hanger + langing gear assembly
    mb = mh*.8/1; %kg, mass of the base plate with the camera
    g = 9.8; %m/s^2, gravitational constant on the surface of Earth
    rd = 1*25.4/1000; % inch to meter conversion, distance from CG of upper to hinge of hanger
    Rd = rd*2;
    rh = 4*25.4/1000; % distance from hanger hinge point to its own CG
    rhm = 1.25*25.4/1000; % distance between the hanger hinge point and the base hinge point
    L = 3.5*25.4/1000;
    rb = (L-L/10); %m, distance from base hinge to base CG
    I1 = md*(rd)^2; % upper moment of inertia
    I3 = mb*(L/10)^2/12; % base moment of intertia
    I2 = mh*(rh)^2/12; % hanger moment of inertia
% System stiffness'
  % Cubic joint
    Ckt1 = 74.55; %N-m/rad^3, angular stiffness of the UAV frame spring/damper
    mt = mh+mb; %kg
    rt = rb; %m
    It = mt*rt^2;
    ft = 0.75;
    wt = 2*3.14159*ft;
    Ckt2 = wt^2*It; %N-m/rad
    beta = .51;
    bt = beta*2*sqrt(Ckt2*It); %Critically damped for beta = 1
  % bearing friction
    bf = 0.015;
  % KMM frequency
    fn = 150;
    wn = 2*3.14159*fn;
    mtot = 2*md;
    Itot = 2*I2;
    k_kmm = mtot*wn^2;
    k_kmmt = Itot*wn^2; % only used to allow for flow input, if testing with no input then set this value low (0.001)
  % KMM joint damping coeffs
    b_kmm = 2*sqrt(k_kmm*mtot);
    b_kmmt = 2*sqrt(k_kmmt*Itot);
% check max sys freq and use for time step determination    
w_max = wn;

  %Controller Gains
    Kp = 0;%25.20;
    Kd = Kp/20;
    Ki = Kd/1;
    
% coeffs for adding stiffness; used while testing code    
G1 = 1;
G2 = G1;
G3 = G2;

%% Define initial conditions
init_ang1 = 10*3.14159/180;
init_ang2 = 10*3.14159/180;
init_ang3 = 10*3.14159/180;
initial = [0 0 1/k_kmm*md*g 0 1/k_kmm*mh*g 0 1/k_kmm*mb*g 0 0 0 0 0 0 0 0 0 0 init_ang1 init_ang2 init_ang3 0 0];     

%% Define time vector
tstart = 0;
tfin = 2.5;
tsteps = 100*tfin;
dt = 1/(10*(w_max/(2*3.14)));
tspan = (tstart:dt:tfin);%linspace(tstart,tfin,tsteps);

%% Call ode15s (stiff systems) to solve ODEs for system
[t,s] = ode15s(@eqns_hangerControl_022213,tspan,initial);
ds=zeros(length(t),22);
ext=zeros(length(t),16);
%% Get derivatives and outputs for system using the ODE solution matrix [S]
for i=1:length(t)
    [ds(i,:) ext(i,:)] = eqns_hangerControl_022213(t(i), s(i,:));
end
% Compute variables with new units and give physical names
upper_angle = s(:,18)*180/3.14159;                      % degrees
upper_angVel = ds(:,18)*180/3.14159;                    % degrees/sec
control_torque = ext(:,1)*(100/(g+.001));               % kg-cm   
hanger_angle = s(:,19)*180/3.14159;                     % degrees
hanger_angVel = ds(:,19)*180/3.14159;                   % degrees/sec
base_angle = s(:,20)*180/3.14159;                     % degrees
base_angVel = ds(:,20)*180/3.14159;                   % degrees/sec
q3 = s(:,5);
q4 = s(:,6);
qslop = s(:,2);
tau_slop = ext(:,16)*100/(g+0.001); % kg-cm, torque

%% Plotting and animations
fs = 14;
% figure(1)
% suptitle('10 degree Initial Displacement')
% subplot(3,1,1); [AX,H1,H2]=plotyy(tspan,control_torque,tspan,upper_angle);set(gca,'FontSize',fs);
% title('Control torque, Upper Angle');xlabel('Time [s]');
% set(get(AX(1),'Ylabel'),'String','Control Torque [kg-cm]'); 
% set(get(AX(2),'Ylabel'),'String','Angle [degrees]'); 
% set(H1,'LineStyle','-');
% set(H2,'LineStyle','--');
% set(H1,'Color','k');
% set(H2,'Color','k');
% legend('Control Torque','Upper Angle');
% subplot(3,1,2); [BX,H11,H21]=plotyy(tspan,control_torque,tspan,hanger_angVel);set(gca,'FontSize',fs);
% title('Control torque, Hanger Angular Velocity');xlabel('Time [s]');
% set(get(BX(1),'Ylabel'),'String','Control Torque [kg-cm]'); 
% set(get(BX(2),'Ylabel'),'String','Ang. Vel. [degrees/sec]');
% set(H11,'LineStyle','-');
% set(H21,'LineStyle','--');
% set(H11,'Color','k');
% set(H21,'Color','k');
% legend('Control Torque','Hanger Angular Velocity');
% wtn = sqrt(g/(rh*.8));
% for i = 1:length(tspan)
% theta_test(i,1) = init_ang3*cos(wtn*tspan(i))*180/3.14159;
% end
% subplot(3,1,3); plot(tspan,upper_angle,'-k',tspan,hanger_angle,'--k',tspan,base_angle,'-.k');set(gca,'FontSize',fs);%,tspan,theta_test,'*-k');
% title('Upper, Hanger, and Base angular displacements');xlabel('Time [s]');ylabel('Angle [degrees]');
% legend('Upper Angle','Hanger Angle','Base Angle');
figure(1)
% % % suptitle('10 degree Initial Displacement')
% % subplot(3,1,1); plot(tspan,control_torque,tspan,upper_angle);set(gca,'FontSize',fs);
% % title('Upper Angle Displacement');xlabel('Time [s]');
% % %legend('Control Torque','Upper Angle');
% % subplot(3,1,2); plot(tspan,control_torque,tspan,hanger_angVel);set(gca,'FontSize',fs);
% % title('Control torque, Hanger Angular Velocity');xlabel('Time [s]');
% % %legend('Control Torque','Hanger Angular Velocity');
% subplot(1,2,1)
plot(tspan,upper_angle,'-k',tspan,hanger_angle,'--k',tspan,base_angle,'-.k');set(gca,'FontSize',fs);%,tspan,theta_test,'*-k');
title('Upper, Hanger, and Base angular displacements');xlabel('Time [s]');ylabel('Angle [degrees]');
legend('Upper Angle','Hanger Angle','Base Angle');
% % %%
% figure(1)
% suptitle('10 degree Initial Displacement')
% subplot(1,3,1); plot(tspan,ext(:,5),'-k',tspan,ext(:,6),'--k');set(gca,'FontSize',fs);
% title('Restraint Forces');xlabel('Time [s]');ylabel('Force [N]');legend('Fx2','Fy2');
% subplot(1,3,2); plot(tspan,q3,'-k',tspan,q4,'--k');set(gca,'FontSize',fs);
% title('Restraint Displacements');xlabel('Time [s]');ylabel('Displacement [m]');legend('dx2','dy2');
% subplot(1,3,3); plot(tspan,qslop*180/3.14159,'-k');set(gca,'FontSize',fs);
% title('Bushing Joint Relative Displacement');xlabel('Time [s]');ylabel('Angle [degress]');
% subplot(1,2,2)
% set(gca,'FontSize',fs);
% plot(tspan,control_torque,'-k');title('Control Torque');xlabel('Time [s]');ylabel('Torque [kg-cm]');
%%
% Points for UAV body
UAVa_x = zeros(length(tspan),1);UAVa_y = zeros(length(tspan),1);
UAVb_x = Rd*cos(s(:,18));UAVb_y = Rd*sin(s(:,18));
% Intermediate points
c_xn = rhm*cos(s(:,19));c_yn = rhm*sin(s(:,19));
c_x = c_xn + UAVb_x; c_y = c_yn + UAVb_y;
% Points for Landing Gear body
LGb_xn = rh*cos(s(:,19)); LGb_yn = rh*sin(s(:,19));
LGb_x = c_x + LGb_xn; LGb_y = c_y + LGb_yn;
% Points for Camera body
Cb_xn = rb*cos(s(:,20));Cb_yn = rb*sin(s(:,20));
Cb_x = Cb_xn + c_x; Cb_y = Cb_yn + c_y;
% % Distances; check that they are constant
% LG_dist = sqrt((LGb_x - UAVb_x).^2 + (LGb_y - UAVb_y).^2); 
% Cb_dist = sqrt((Cb_x - c_x).^2 + (Cb_y - c_y).^2); 
% 
% figure(4)
% plot(tspan,LG_dist,tspan,Cb_dist);
% 
% UAV body line
% xmin = -.2; xmax = .2;
% ymin = -.2; ymax = 0;
% name1 = 'sine-ramp_tight_w-ctrl_short.avi';
% name2 = 'sine-ramp_loose_w-ctrl_short.avi';
% name3 = 'sine-ramp_loose_no-ctrl_short.avi';
% vid = VideoWriter(name3);
% vid.Quality = 100;
% vid.FrameRate = 15;
% open(vid);
% for i = 1:80:length(tspan)
%     close
%     figure(3)
%     %title('Tight Bushings, Sequence Input, With Control');
%     title('Loose Bushings, Sequence Input, No Control');
%     %title('Loose Bushings, Sequence Input, With Control');
%     xlabel('meters');ylabel('meters');
%     axis([xmin xmax ymin ymax]);
%     line([UAVa_y(i) UAVb_y(i)],[-UAVa_x(i) -UAVb_x(i)],'LineStyle','-','Color','k','LineWidth',4);
%     line([UAVb_y(i) LGb_y(i)],[-UAVb_x(i) -LGb_x(i)],'LineStyle','-','Color','b','LineWidth',4);
%     line([c_y(i) Cb_y(i)],[-c_x(i) -Cb_x(i)],'LineStyle','-','Color','r','LineWidth',4);
%     writeVideo(vid, getframe(gcf));
% end
% close(vid);
%winopen(name1);
% figure(2)
% set(gca,'FontSize',16);
% plot(tspan,theta_test,'--k',tspan,base_angle,'-k');
% title('Simple Pendulum Comparison');xlabel('Time [s]');ylabel('Angular Displacement [degrees]');
% legend('Analytical Model','Numerical Model');
% figure(2)
% set(gca,'FontSize',16);
% plot(tspan,base_angle,'-k');
% title('Testing the Bearing Friction');xlabel('Time [s]');ylabel('Angular Displacement [degrees]');
% %legend('Numerical Model');
% x1=ext(:,7);
% y1=ext(:,8);
% x2=ext(:,9);
% y2=ext(:,10);

% figure(2)
% plot(tspan,tau_slop);

% plot(tspan,y2,tspan,y1,tspan,R1*sin(s(:,12)));
% legend('2','1','test');


toc